function imgStack = read_img_stack_from_file(fileName, bitAlignDiv)

%
% function imgStack = read_img_stack_from_file(fileName, bitAlignDiv)
%
% This function reads an image stack from a file using the 'imread'
% function. If the image data was modified due to the MSB alignment during
% the saving of the file, a denominator is passed to the function to
% obtain the original values.
%
% Input parameters:
% fileName:    Image stack file name
% bitAlignDiv: Denominator for bit alignment correction
%
% Return value:
% imgStack: 3D image stack
%

    info = imfinfo(fileName);
    nImages = numel(info);
    imgStack = [];

    for iImage = 1:nImages
        img = imread(fileName, iImage, 'Info', info);
        imgStack(:, :, iImage) = img / bitAlignDiv;
    end
end
