function [mLifetime pLifetime] = flim_get_lifetimes(modIndex, phase, f)

%
% function [mLifetime pLifetime] = flim_get_lifetimes(modIndex, phase, f)
%
% This function computes the lifetimes based on the modulation index and
% phase at a given modulation frequency.
%
% Input parameters:
% modIndex: 2D modulation index image
% phase:    2D phase image (in radians)
% f:        Modulation frequency (in Hertz)
%
% Return values:
% mLifetime: 2D lifetime image based on the modulation index (in seconds)
% pLifetime: 2D lifetime image based on the phase (in seconds)
%

    mLifetime =  (1 / (2 * pi * f)) * sqrt(1 ./ modIndex .^ 2 - 1);
    pLifetime = -(1 / (2 * pi * f)) * tan(phase);

end
