function [modIndex phase] = flim_referencing(modIndexSample, ...
    phaseSample, modIndexRef, phaseRef, tauRef, f)

%
% function [modIndex phase] = flim_referencing(modIndexSample, ...
%     phaseSample, modIndexRef, phaseRef, tauRef, f)
%
% This function references a sample measurement by a reference measurement.
% The reference sample is assumed to be an ideal first-order lowpass
% system whose behaviour is characterized by its time constant tauRef at a
% specific modulation frequency f.
%
% Input parameters:
% modIndexSample: 2D modulation index image of sample measurement
% phaseSample:    2D phase image of sample measurement
% modIndexRef:    2D modulation index image of reference measurement
% phaseRef:       2D phase image of reference measurement
% tauRef:         Time constant of reference system (in seconds)
% f:              Modulation frequency (in Hertz)
%
% Return values:
% modIndex: Referenced 2D modulation index image
% phase:    Referenced 2D phase image
%

    modIndex = (1 / sqrt(1 + (2 * pi * f * tauRef) ^ 2)) * ...
               modIndexSample ./ modIndexRef;

    phase = phaseSample - phaseRef - atan(2 * pi * f * tauRef);

end
